package com.qmth.ops.api.controller.ai;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.core.ai.model.AiConstants;
import com.qmth.boot.core.ai.model.llm.*;
import com.qmth.boot.core.exception.ForbiddenException;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.ops.api.security.AccessOrg;
import com.qmth.ops.biz.domain.LlmOrgConfig;
import com.qmth.ops.biz.service.LlmClientService;
import com.qmth.ops.biz.service.LlmOrgConfigService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@Aac(auth = true, signType = SignatureType.SECRET)
public class LlmController {

    @Resource
    private LlmOrgConfigService llmOrgConfigService;

    @Resource
    private LlmClientService llmClientService;

    @PostMapping(AiConstants.LLM_CHAT_PATH)
    public ChatResult chat(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type,
            @RequestBody @Validated ChatRequest request) throws Exception {
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg(), type);
        if (config != null && config.getLeftCount() > 0) {
            ChatResult result = llmClientService.chat(request, config.getModelId());
            llmOrgConfigService.consume(config);
            return result;
        } else {
            throw new ForbiddenException(
                    "Chat api is disabled or exhausted for org=" + accessOrg.getOrg().getCode() + ", app type=" + type);
        }
    }

    @PostMapping(AiConstants.LLM_BALANCE_PATH)
    public LlmAppBalance balance(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type) {
        LlmOrgConfig config = llmOrgConfigService.findByOrgAndAppType(accessOrg.getOrg(), type);
        LlmAppBalance balance = new LlmAppBalance();
        if (config != null) {
            balance.setPermitCount(config.getPermitCount());
            balance.setLeftCount(config.getLeftCount());
        }
        return balance;
    }

    @PostMapping(AiConstants.LLM_PROMPT_TEMPLATE_PATH)
    public PromptTemplate getPromptTemplate(@RequestAttribute AccessOrg accessOrg,
            @RequestHeader(AiConstants.LLM_APP_TYPE_HEADER) LlmAppType type) {
        return null;
    }
}
